--[[

	### GESTURES ###
 
	File :			Mells_Gestures.lua
	Date :			2013.10.03
	Author : 		Mells
	Description : 	Simple Gestures Class to recognize Swipes on a Sprite, and easily add callbacks.
					Inspired by different classes shared by the Gideros community.
]]--

Gestures = Core.class(EventDispatcher)

-- ----------------------
-- Output true > print Swipe types in the console
-- ----------------------
local output = false
 
function Gestures:init(parent, options)
 
	self.parent = parent
 
	-- -----------------------
	-- Setup Variables
	-- -----------------------
	local conf = {
		maxDuration = 0.5,
		minDistance = 70		
	}
	if options then
		for i, option in pairs(options) do
			conf[i] = option
		end	
	end
 
	self.options = conf

	-- -----------------------
	-- Add Event Listeners
	-- -----------------------
	parent:addEventListener(Event.TOUCHES_BEGIN, self.onTouchBegin, self)
	parent:addEventListener(Event.TOUCHES_MOVE, self.onTouchMove, self)
	parent:addEventListener(Event.TOUCHES_END, self.onTouchEnd, self)
 
	-- -----------------------
	-- Set Active
	-- -----------------------
	self:setActive(true)

	-- Add Native callbacks :
	-- Accepted : onSwipeUp, onSwipeDown, onSwipeLeft, onSwipeRight, onSwipeUpLeft, onSwipeUpRight, onSwipeDownLeft, onSwipeRight	
	local swipeCallbacks = {"onSwipeUp", "onSwipeDown", "onSwipeLeft", "onSwipeRight", "onSwipeUpLeft", "onSwipeUpRight", "onSwipeDownLeft", "onSwipeDownRight"}
	local swipeCallback
 
	print ("------------------")
	print ("Add Callbacks")
	print ("------------------")
	for i=1, #swipeCallbacks, 1 do
		swipeCallback = swipeCallbacks[i]
		self[swipeCallback] = function(callback) print ("Default ["..callback.."]") end		
		self:addCallback(swipeCallback,  self[swipeCallback])
	end
 
end
 
-- ------------------------------------
 
function Gestures:setActive(bool)
	self.active = bool
end
 
-- ------------------------------------
 
function Gestures:isActive()
	if self.active == true then return true end
	return false
end
 
-- ------------------------------------
 
function Gestures:onTouchBegin(event)
 
	-- Is Active?
	if not(self:isActive()) then return end
	if not self.parent:hitTestPoint(event.touch.x, event.touch.y) then return end
 
	if self.timer then
		self.timer:stop()
	end
	self.swipeDirection = nil

	self.startX = event.touch.x
	self.startY = event.touch.y
	self.lastX = event.touch.x
	self.lastY = event.touch.y
	self.activeTouchId = event.touch.id
 
	-- --------------------
	-- Duration &amp; Distance
	-- --------------------
	self.swipeDuration = 0
 
	-- -------------
	-- Add timer
	-- -------------
	local timer = Timer.new(100, 10)
	self.timer = timer
	timer:addEventListener(Event.TIMER, self.onSwipeTimer, self)
	timer:addEventListener(Event.TIMER_COMPLETE, function() self.timer:stop() self.canSwipe = false end)
	timer:start()	
 
	-- -----------
	-- CALLBACK
	-- -----------
	self:runCallback(event:getType(), event)
 
	-- -----------
	-- Stop Propagation	
	-- -----------
	event:stopPropagation()	
 
end
 
-- ----------------------------------------
 
function Gestures:onTouchMove(event)
 
	self.lastX = event.touch.x
	self.lastY = event.touch.y
 
	-- -----------
	-- CALLBACK
	-- -----------
	self:runCallback(event:getType(), event)
 
	-- -----------
	-- Stop Propagation
	-- -----------
	event:stopPropagation()
 
end

-- ----------------------------------------
 
function Gestures:onTouchEnd(event)
 
	local options = self.options
	local up, down, left, right
 
	-- ------------------------
	-- Check 
	-- ------------------------
	if self.activeTouchId == event.touch.id then
 
		self.timer:stop()
		self.timer = nil		
 
		-- ------------------------
		-- Check Swipe Duration
		-- ------------------------
		if self.swipeDuration < options.maxDuration*0.99 then
 
				-- ----------------------
				-- GET SWIPE DIRECTION
				-- ----------------------				
				-- LEFT
				if(event.touch.x < self.startX - options.minDistance) then					
					left = true
				-- RIGHT	
				elseif(event.touch.x > self.startX + options.minDistance) then
					right = true								
				end
 
				-- UP
				if(event.touch.y < self.startY - options.minDistance) then					
					up = true
				-- DOWN
				elseif(event.touch.y > self.startY + options.minDistance) then
					down = true									
				end
 
				-- ----------------------
				-- SET SWIPE DIRECTION
				-- ----------------------				
				if up then				
					if left then self.swipeDirection = "onSwipeUpLeft"
					elseif right then self.swipeDirection = "onSwipeUpRight"
					else self.swipeDirection = "onSwipeUp"						
					end
 
				elseif down then				
					if left then self.swipeDirection = "onSwipeDownLeft"
					elseif right then self.swipeDirection = "onSwipeDownRight"
					else self.swipeDirection = "onSwipeDown"
					end
				elseif left then self.swipeDirection = "onSwipeLeft"
				elseif right then self.swipeDirection = "onSwipeRight"
				end
 
				-- -----------
				-- CALLBACK
				-- -----------
				self:runCallback(self.swipeDirection, event, {duration = self.swipeDuration, start = {self.startX, self.startY}})				
		end
 
		-- ----------------------
		-- Reset
		-- ----------------------
		self.swipeDuration = 0
		self.startX = nil
		self.startY = nil
		self.lastX = nil
		self.lastY = nil
		self.activeTouchId = nil		
 
		-- -----------
		-- CALLBACK
		-- -----------
		self:runCallback(event:getType(), event)
 
		-- -----------
		-- Stop Propagation
		-- -----------
		event:stopPropagation()
 
	end
 
end

-- ----------------------------------------
 
function Gestures:onSwipeTimer(event)
	self.swipeDuration = self.swipeDuration + .1
end

-- -------------------------------------------
-- Accepted : "touchesBegin", "touchesMove", "touchesEnd"
function Gestures:addCallback(id, callback)
	if output then print ("Add Callback : ", id) end
	self.callbacks = self.callbacks or {}
	self.callbacks[id] = callback		
end
 
-- -------------------------------------------
 
function Gestures:getCallbacks()
	return self.callbacks
end
 
-- -------------------------------------------
 
function Gestures:removeCallbacks(id)
	print ("------------------")
	print ("Remove callbacks")
	print ("------------------")
	local callbacks = self:getCallbacks()
	for i, callback in pairs(callbacks) do
		self:removeCallback(i)
	end	
	callback = nil
end
 
-- -------------------------------------------
 
function Gestures:removeCallback(id)
	local callbacks = self:getCallbacks()
	local flag
	for i, callback in pairs(callbacks) do		
		if id == i then flag = i end
	end
 
	if flag then
		if output then print ("Remove Callback : ", flag) end
		self[flag] = nil
		callbacks[flag] = nil
	end
end

-- ----------------------------------------
 
function Gestures:runCallback(type, event, options)	
	local callbacks = self:getCallbacks()
 
	-- -----------
	-- CALLBACK
	-- -----------
	if self.callbacks and self.callbacks[type] then self.callbacks[type](type, event, options) end
end
 
-- -------------------------------------------
 
function Gestures:free()	
	local timer = self.timer	
	if timer then
		timer:removeEventListener(Event.TIMER, self.onSwipeTimer, self)
		timer:removeEventListener(Event.TIMER_COMPLETE, function() self.timer:stop() self.canSwipe = false end)
		timer:stop()
		timer = nil
	end	
 
	self:removeCallbacks()	
end